@@ -1,7 +1,6 @@
-# library(testthat)
+library(testthat)
 context("Untangle two dendrograms for plotting a tanglegram")
 
-
 test_that("shuffle works", {
   suppressWarnings(RNGversion("3.5.0"))
   #    library(magrittr)
@@ -113,6 +112,35 @@ test_that("untangle_step_rotate_2side work", {
 
 
 
+test_that("untangle_step_rotate_both_side work", {
+   suppressWarnings(RNGversion("3.5.0"))
+   # Entanglement should be zero after applying algorithm, per Fig. 4 of 'Shuffle & untangle: novel untangle methods for solving the tanglegram layout problem' (Nguyen et al. 2022)
+   example_labels <- c("Versicolor 90", "Versicolor 54", "Versicolor 81", "Versicolor 63", "Versicolor 72", "Versicolor 99", "Virginica 135", "Virginica 117", "Virginica 126", "Virginica 108", "Virginica 144", "Setosa 27", "Setosa 18", "Setosa 36", "Setosa 45", "Setosa 9")
+   # library(dplyr)
+   # iris_modified <- 
+   #    datasets::iris %>%
+   #    mutate(Row = row_number()) %>%
+   #    mutate(Label = paste(str_to_title(Species), Row)) %>%
+   #    dplyr::filter(Label %in% example_labels)
+   iris_modified <- datasets::iris
+   iris_modified$Row <- seq_len(nrow(iris_modified))
+   iris_modified$Label <- paste(tools::toTitleCase(as.character(iris_modified$Species)), iris_modified$Row)
+   iris_modified <- iris_modified[iris_modified$Label %in% example_labels, ]
+   
+   iris_numeric <- iris_modified[,1:4]
+   rownames(iris_numeric) <- iris_modified$Label
+   
+   dend1 <- as.dendrogram(hclust(dist(iris_numeric), method = "single"))
+   dend2 <- as.dendrogram(hclust(dist(iris_numeric), method = "complete"))
+   result <- untangle_step_rotate_both_side(dend1, dend2)
+   dend1 <- result[[1]]
+   dend2 <- result[[2]]
+   expect_identical(entanglement(dend1, dend2, L = 2), 0)
+   
+})
+
+
+
 
 #
 # # This one is an example of how 2side rotation is better. But it takes longer
@@ -187,3 +215,47 @@ test_that("untangle (main function) works for 2 step", {
   out2 <- untangle_step_rotate_2side(dend1, dend2)
   expect_identical(out1, out2)
 })
+
+
+
+
+test_that("untangle_best_k_to_rotate_by_1side works", {
+   suppressWarnings(RNGversion("3.5.0"))
+   set.seed(226)
+   
+   dend1 <- USArrests[1:10, ] %>%
+      dist() %>%
+      hclust() %>%
+      as.dendrogram()
+   dend2 <- shuffle(dend1)
+   original_entanglement <- entanglement(dend1, dend2)
+   expect_identical(round(original_entanglement, 3), 0.248)
+   # resolve entanglement
+   dend1_corrected <- untangle_best_k_to_rotate_by_1side(dend1, dend2)
+   corrected_entanglement <- entanglement(dend1_corrected, dend2)
+   
+   # reduces entanglement from 0.248 to 0
+   expect_identical(round(corrected_entanglement, 3), 0)
+})
+
+
+
+
+test_that("untangle_best_k_to_rotate_by_2side_backNforth works", {
+   suppressWarnings(RNGversion("3.5.0"))
+   set.seed(30) 
+   
+   dend1 <- USArrests[1:10, ] %>%
+      dist() %>%
+      hclust() %>%
+      as.dendrogram()
+   dend2 <- shuffle(dend1)
+   original_entanglement <- entanglement(dend1, dend2)
+   expect_identical(round(original_entanglement, 3), 0.251)
+   # resolve entanglement
+   dends_corrected <- untangle_best_k_to_rotate_by_2side_backNforth(dend1, dend2, L = 1, print_times = F)
+   corrected_entanglement <- entanglement(dends_corrected[[1]], dends_corrected[[2]])
+   
+   # reduces entanglement from 0.251 to 0
+   expect_identical(round(corrected_entanglement, 3), 0)
+})
\ No newline at end of file
